@@ -1,20 +1,9 @@
-// MediaInfo_Config - Configuration class
-// Copyright (C) 2005-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
+
 //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 //
 // Global configuration of MediaInfo
@@ -29,6 +18,97 @@
 #endif
 //---------------------------------------------------------------------------
 
+//---------------------------------------------------------------------------
+// Debug
+#ifdef MEDIAINFO_DEBUG
+    #include <stdio.h>
+    #include <windows.h>
+    namespace MediaInfo_Config_Debug
+    {
+        FILE* F;
+        std::string Debug;
+        SYSTEMTIME st_In;
+
+        void Debug_Open(bool Out)
+        {
+            F=fopen("C:\\Temp\\MediaInfo_Debug.txt", "a+t");
+            Debug.clear();
+            SYSTEMTIME st;
+            GetLocalTime( &st );
+
+            char Duration[100];
+            if (Out)
+            {
+                FILETIME ft_In;
+                if (SystemTimeToFileTime(&st_In, &ft_In))
+                {
+                    FILETIME ft_Out;
+                    if (SystemTimeToFileTime(&st, &ft_Out))
+                    {
+                        ULARGE_INTEGER UI_In;
+                        UI_In.HighPart=ft_In.dwHighDateTime;
+                        UI_In.LowPart=ft_In.dwLowDateTime;
+
+                        ULARGE_INTEGER UI_Out;
+                        UI_Out.HighPart=ft_Out.dwHighDateTime;
+                        UI_Out.LowPart=ft_Out.dwLowDateTime;
+
+                        ULARGE_INTEGER UI_Diff;
+                        UI_Diff.QuadPart=UI_Out.QuadPart-UI_In.QuadPart;
+
+                        FILETIME ft_Diff;
+                        ft_Diff.dwHighDateTime=UI_Diff.HighPart;
+                        ft_Diff.dwLowDateTime=UI_Diff.LowPart;
+
+                        SYSTEMTIME st_Diff;
+                        if (FileTimeToSystemTime(&ft_Diff, &st_Diff))
+                        {
+                            sprintf(Duration, "%02hd:%02hd:%02hd.%03hd", st_Diff.wHour, st_Diff.wMinute, st_Diff.wSecond, st_Diff.wMilliseconds);
+                        }
+                        else
+                            strcpy(Duration, "            ");
+                    }
+                    else
+                        strcpy(Duration, "            ");
+
+                }
+                else
+                    strcpy(Duration, "            ");
+            }
+            else
+            {
+                st_In=st;
+                strcpy(Duration, "            ");
+            }
+
+            fprintf(F,"                                       %02hd:%02hd:%02hd.%03hd %s", st.wHour, st.wMinute, st.wSecond, st.wMilliseconds, Duration);
+        }
+
+        void Debug_Close()
+        {
+            Debug += "\r\n";
+            fwrite(Debug.c_str(), Debug.size(), 1, F); \
+            fclose(F);
+        }
+    }
+    using namespace MediaInfo_Config_Debug;
+
+    #define MEDIAINFO_DEBUG1(_NAME,_TOAPPEND) \
+        Debug_Open(false); \
+        Debug+=", ";Debug+=_NAME; \
+        _TOAPPEND; \
+        Debug_Close();
+
+    #define MEDIAINFO_DEBUG2(_NAME,_TOAPPEND) \
+        Debug_Open(true); \
+        Debug+=", ";Debug+=_NAME; \
+        _TOAPPEND; \
+        Debug_Close();
+#else // MEDIAINFO_DEBUG
+    #define MEDIAINFO_DEBUG1(_NAME,__TOAPPEND)
+    #define MEDIAINFO_DEBUG2(_NAME,__TOAPPEND)
+#endif // MEDIAINFO_DEBUG
+
 //---------------------------------------------------------------------------
 #include "MediaInfo/Setup.h"
 //---------------------------------------------------------------------------
@@ -45,8 +125,8 @@ namespace MediaInfoLib
 {
 
 //---------------------------------------------------------------------------
-const Char*  MediaInfo_Version=__T("MediaInfoLib - v0.7.62");
-const Char*  MediaInfo_Url=__T("http://mediainfo.sourceforge.net");
+const Char*  MediaInfo_Version=__T("MediaInfoLib - v0.7.83");
+const Char*  MediaInfo_Url=__T("http://MediaArea.net/MediaInfo");
       Ztring EmptyZtring;       //Use it when we can't return a reference to a true Ztring
 const Ztring EmptyZtring_Const; //Use it when we can't return a reference to a true Ztring, const version
 const ZtringListList EmptyZtringListList_Const; //Use it when we can't return a reference to a true ZtringListList, const version
@@ -67,6 +147,7 @@ void MediaInfo_Config_CodecID_Audio_Riff      (InfoMap &Info);
 void MediaInfo_Config_CodecID_Text_Matroska   (InfoMap &Info);
 void MediaInfo_Config_CodecID_Text_Mpeg4      (InfoMap &Info);
 void MediaInfo_Config_CodecID_Text_Riff       (InfoMap &Info);
+void MediaInfo_Config_CodecID_Other_Mpeg4     (InfoMap &Info);
 void MediaInfo_Config_Codec                   (InfoMap &Info);
 void MediaInfo_Config_DefaultLanguage         (Translation &Info);
 void MediaInfo_Config_Iso639_1                (InfoMap &Info);
@@ -115,7 +196,7 @@ void MediaInfo_Config::Init()
         InitDataNotRepeated_GiveUp=false;
     #endif //MEDIAINFO_ADVANCED
     MpegTs_MaximumOffset=64*1024*1024;
-    MpegTs_MaximumScanDuration=15000000000LL;
+    MpegTs_MaximumScanDuration=30000000000LL;
     MpegTs_ForceStreamDisplay=false;
     #if MEDIAINFO_ADVANCED
         MpegTs_VbrDetection_Delta=0;
@@ -135,10 +216,12 @@ void MediaInfo_Config::Init()
     ParseSpeed=(float32)0.5;
     Verbosity=(float32)0.5;
     Trace_Level=(float32)0.0;
+    Compat=70778;
     Trace_TimeSection_OnlyFirstOccurrence=false;
     Trace_Format=Trace_Format_Tree;
     Language_Raw=false;
     ReadByHuman=true;
+    Legacy=true;
     LegacyStreamDisplay=true;
     SkipBinaryData=false;
     Demux=0;
@@ -168,7 +251,9 @@ void MediaInfo_Config::Init()
 
 Ztring MediaInfo_Config::Option (const String &Option, const String &Value_Raw)
 {
+    CS.Enter();
     SubFile_Config(Option)=Value_Raw;
+    CS.Leave();
 
     String Option_Lower(Option);
     size_t Egal_Pos=Option_Lower.find(__T('='));
@@ -329,6 +414,15 @@ Ztring MediaInfo_Config::Option (const String &Option, const String &Value_Raw)
     {
         return ReadByHuman_Get()?__T("1"):__T("0");
     }
+    else if (Option_Lower==__T("legacy"))
+    {
+        Legacy_Set(Value.To_int8u()?true:false);
+        return Ztring();
+    }
+    else if (Option_Lower==__T("legacy_get"))
+    {
+        return Legacy_Get()?__T("1"):__T("0");
+    }
     else if (Option_Lower==__T("legacystreamdisplay"))
     {
         LegacyStreamDisplay_Set(Value.To_int8u()?true:false);
@@ -477,6 +571,8 @@ Ztring MediaInfo_Config::Option (const String &Option, const String &Value_Raw)
     }
     else if (Option_Lower==__T("details")) //Legacy for trace_level
     {
+        if (Value == __T("0"))
+            Trace_Level=0;
         return MediaInfo_Config::Option(__T("Trace_Level"), Value);
     }
     else if (Option_Lower==__T("details_get")) //Legacy for trace_level
@@ -494,6 +590,13 @@ Ztring MediaInfo_Config::Option (const String &Option, const String &Value_Raw)
     else if (Option_Lower==__T("trace_level"))
     {
         Trace_Level_Set(Value);
+        if (Inform_Get()==__T("MAXML"))
+            Trace_Format_Set(Trace_Format_XML); // All must be XML
+        if (Inform_Get()==__T("XML") || Inform_Get()==__T("MAXML"))
+        {
+            Inform_Set(Ztring());
+            Trace_Format_Set(Trace_Format_XML); // TODO: better coherency in options
+        }
         return Ztring();
     }
     else if (Option_Lower==__T("trace_level_get"))
@@ -525,6 +628,8 @@ Ztring MediaInfo_Config::Option (const String &Option, const String &Value_Raw)
         CriticalSectionLocker CSL(CS);
         if (NewValue_Lower==__T("csv"))
             Trace_Format_Set(Trace_Format_CSV);
+        else if (NewValue_Lower==__T("xml") || NewValue_Lower==__T("MAXML"))
+            Trace_Format_Set(Trace_Format_XML);
         else
             Trace_Format_Set(Trace_Format_Tree);
         return Ztring();
@@ -568,7 +673,7 @@ Ztring MediaInfo_Config::Option (const String &Option, const String &Value_Raw)
     }
     else if (Option_Lower==__T("info_parameters_csv"))
     {
-        return Info_Parameters_Get();
+        return Info_Parameters_Get(Value==__T("Complete"));
     }
     else if (Option_Lower==__T("info_codecs"))
     {
@@ -737,6 +842,22 @@ Ztring MediaInfo_Config::Option (const String &Option, const String &Value_Raw)
     {
         return MpegTs_ForceStreamDisplay_Get()?__T("1"):__T("0");
     }
+    else if (Option_Lower==__T("maxml_streamkinds"))
+    {
+        #if MEDIAINFO_ADVANCED
+            return MAXML_StreamKinds_Get();
+        #else // MEDIAINFO_ADVANCED
+            return __T("advanced features are disabled due to compilation options");
+        #endif // MEDIAINFO_ADVANCED
+    }
+    else if (Option_Lower==__T("maxml_fields"))
+    {
+        #if MEDIAINFO_ADVANCED
+            return MAXML_Fields_Get(Value);
+        #else // MEDIAINFO_ADVANCED
+            return __T("advanced features are disabled due to compilation options");
+        #endif // MEDIAINFO_ADVANCED
+    }
     else if (Option_Lower==__T("custommapping"))
     {
         CustomMapping_Set(Value);
@@ -1020,6 +1141,19 @@ bool MediaInfo_Config::ReadByHuman_Get ()
     return ReadByHuman;
 }
 
+//---------------------------------------------------------------------------
+void MediaInfo_Config::Legacy_Set (bool NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    Legacy=NewValue;
+}
+
+bool MediaInfo_Config::Legacy_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return Legacy;
+}
+
 //---------------------------------------------------------------------------
 void MediaInfo_Config::LegacyStreamDisplay_Set (bool NewValue)
 {
@@ -1081,6 +1215,19 @@ float32 MediaInfo_Config::Trace_Level_Get ()
     return Trace_Level;
 }
 
+//---------------------------------------------------------------------------
+void MediaInfo_Config::Compat_Set (int64u NewValue)
+{
+    CriticalSectionLocker CSL(CS);
+    Compat=NewValue;
+}
+
+int64u MediaInfo_Config::Compat_Get ()
+{
+    CriticalSectionLocker CSL(CS);
+    return Compat;
+}
+
 std::bitset<32> MediaInfo_Config::Trace_Layers_Get ()
 {
     CriticalSectionLocker CSL(CS);
@@ -1344,7 +1491,13 @@ Ztring MediaInfo_Config::Language_Get (const Ztring &Value)
     CriticalSectionLocker CSL(CS);
 
     if (Value.find(__T(" / "))==string::npos)
-        return Language.Get(Value);
+    {
+        if (Value.size()<7 || Value.find(__T("/String"))+7!=Value.size())
+            return Language.Get(Value);
+        Ztring Temp(Value);
+        Temp.resize(Value.size()-7);
+        return Language.Get(Temp);
+    }
 
     ZtringList List;
     List.Separator_Set(0, __T(" / "));
@@ -1361,8 +1514,8 @@ Ztring MediaInfo_Config::Language_Get (const Ztring &Value)
 Ztring MediaInfo_Config::Language_Get (const Ztring &Count, const Ztring &Value, bool ValueIsAlwaysSame)
 {
     //Integrity
-    if (Count.empty())
-        return EmptyString_Get();
+    if (Count.empty() || Count.find_first_not_of(__T("0123456789.+-/*() "))!=string::npos)
+        return Count;
 
     //Different Plurals are available or not?
     if (Language_Get(Value+__T("1")).empty())
@@ -1463,9 +1616,17 @@ void MediaInfo_Config::Inform_Set (const ZtringListList &NewValue)
 {
     if (NewValue.Read(0, 0)==__T("Details"))
         Trace_Level_Set(NewValue.Read(0, 1));
+    else if (Trace_Level_Get() && NewValue.Read(0, 0)==__T("XML"))
+    {
+        Trace_Format_Set(Trace_Format_XML); // TODO: better coherency in options
+        return;
+    }
     else
     {
-        Trace_Level_Set(__T("0"));
+        if (NewValue.Read(0, 0)==__T("MAXML"))
+            Trace_Format_Set(Trace_Format_XML); //All must be XML
+        else
+            Trace_Format_Set(Trace_Format_Tree); // TODO: better coherency in options
 
         CriticalSectionLocker CSL(CS);
 
@@ -1649,6 +1810,13 @@ const Ztring &MediaInfo_Config::CodecID_Get (stream_t KindOfStream, infocodecid_
                                         default: ;
                                     }
                                     break;
+            case Stream_Other   :
+                                    switch (Format)
+                                    {
+                                        case InfoCodecID_Format_Mpeg4    : MediaInfo_Config_CodecID_Other_Mpeg4(CodecID[Format][KindOfStream]); break;
+                                        default: ;
+                                    }
+                                    break;
             default: ;
         }
     }
@@ -1719,6 +1887,22 @@ const Ztring MediaInfo_Config::Iso639_Find (const Ztring &Value)
     return Ztring();
 }
 
+//---------------------------------------------------------------------------
+const Ztring MediaInfo_Config::Iso639_Translate (const Ztring Value)
+{
+    Ztring Code(Value);
+    if (Code.size()==3 && !MediaInfoLib::Config.Iso639_1_Get(Code).empty())
+        Code=MediaInfoLib::Config.Iso639_1_Get(Code);
+    if (Code.size()>3 && !MediaInfoLib::Config.Iso639_Find(Code).empty())
+        Code=MediaInfoLib::Config.Iso639_Find(Code);
+    if (Code.size()>3)
+        return Value;
+    Ztring Language_Translated=MediaInfoLib::Config.Language_Get(__T("Language_")+Code);
+    if (Language_Translated.find(__T("Language_"))==0)
+        return Value; //No translation found
+    return Language_Translated;
+}
+
 //---------------------------------------------------------------------------
 const Ztring &MediaInfo_Config::Info_Get (stream_t KindOfStream, const Ztring &Value, info_t KindOfInfo)
 {
@@ -1796,9 +1980,9 @@ const ZtringListList &MediaInfo_Config::Info_Get(stream_t KindOfStream)
 }
 
 //---------------------------------------------------------------------------
-Ztring MediaInfo_Config::Info_Parameters_Get ()
+Ztring MediaInfo_Config::Info_Parameters_Get (bool Complete)
 {
-    CriticalSectionLocker CSL(CS);
+    CS.Enter();
 
     //Loading all
     MediaInfo_Config_General(Info[Stream_General]);
@@ -1820,12 +2004,22 @@ Ztring MediaInfo_Config::Info_Parameters_Get ()
         for (size_t Pos=0; Pos<Info[StreamKind].size(); Pos++)
             if (!Info[StreamKind].Read(Pos, Info_Name).empty())
             {
-                ToReturn(ToReturn_Pos, 0)=Info[StreamKind].Read(Pos, Info_Name);
-                ToReturn(ToReturn_Pos, 1)=Info[StreamKind].Read(Pos, Info_Info);
+                if (Complete)
+                    ToReturn.push_back(Info[StreamKind].Read(Pos));
+                else
+                {
+                    ToReturn(ToReturn_Pos, 0)=Info[StreamKind].Read(Pos, Info_Name);
+                    ToReturn(ToReturn_Pos, 1)=Info[StreamKind].Read(Pos, Info_Info);
+                }
                 ToReturn_Pos++;
             }
         ToReturn_Pos++;
     }
+    CS.Leave();
+
+    //Reset of language file
+    Language_Set(Ztring()); //TODO: it is reseted to English, it should actually not modify the language config (MediaInfo_Config_xxx() modifies the language config)
+
     return ToReturn.Read();
 }
 
@@ -2021,6 +2215,77 @@ bool MediaInfo_Config::MpegTs_ForceStreamDisplay_Get ()
     return MpegTs_ForceStreamDisplay;
 }
 
+#if MEDIAINFO_ADVANCED
+Ztring MediaInfo_Config::MAXML_StreamKinds_Get ()
+{
+    ZtringList List;
+    CriticalSectionLocker CSL(CS);
+    for (size_t StreamKind=0; StreamKind<Stream_Max; StreamKind++)
+    {
+        if (Info[StreamKind].empty())
+        {
+            switch (StreamKind)
+            {
+                case Stream_General :   MediaInfo_Config_General(Info[Stream_General]);   Language_Set(Stream_General); break;
+                case Stream_Video :     MediaInfo_Config_Video(Info[Stream_Video]);       Language_Set(Stream_Video); break;
+                case Stream_Audio :     MediaInfo_Config_Audio(Info[Stream_Audio]);       Language_Set(Stream_Audio); break;
+                case Stream_Text :      MediaInfo_Config_Text(Info[Stream_Text]);         Language_Set(Stream_Text); break;
+                case Stream_Other :     MediaInfo_Config_Other(Info[Stream_Other]);       Language_Set(Stream_Other); break;
+                case Stream_Image :     MediaInfo_Config_Image(Info[Stream_Image]);       Language_Set(Stream_Image); break;
+                case Stream_Menu :      MediaInfo_Config_Menu(Info[Stream_Menu]);         Language_Set(Stream_Menu); break;
+                default:;
+            }
+        }
+        List.push_back(Info[StreamKind](__T("StreamKind"), Info_Name, Info_Text));
+    }
+
+    List.Separator_Set(0, __T(","));
+    return List.Read();
+}
+#endif // MEDIAINFO_ADVANCED
+
+#if MEDIAINFO_ADVANCED
+extern Ztring Xml_Name_Escape_0_7_78 (const Ztring &Name);
+Ztring MediaInfo_Config::MAXML_Fields_Get (const Ztring &StreamKind_String)
+{
+    CriticalSectionLocker CSL(CS);
+
+    size_t StreamKind=Stream_General;
+    for (; StreamKind<Stream_Max; StreamKind++)
+    {
+        if (Info[StreamKind].empty())
+        {
+            switch (StreamKind)
+            {
+                case Stream_General :   MediaInfo_Config_General(Info[Stream_General]);   Language_Set(Stream_General); break;
+                case Stream_Video :     MediaInfo_Config_Video(Info[Stream_Video]);       Language_Set(Stream_Video); break;
+                case Stream_Audio :     MediaInfo_Config_Audio(Info[Stream_Audio]);       Language_Set(Stream_Audio); break;
+                case Stream_Text :      MediaInfo_Config_Text(Info[Stream_Text]);         Language_Set(Stream_Text); break;
+                case Stream_Other :     MediaInfo_Config_Other(Info[Stream_Other]);       Language_Set(Stream_Other); break;
+                case Stream_Image :     MediaInfo_Config_Image(Info[Stream_Image]);       Language_Set(Stream_Image); break;
+                case Stream_Menu :      MediaInfo_Config_Menu(Info[Stream_Menu]);         Language_Set(Stream_Menu); break;
+                default:;
+            }
+        }
+        if (StreamKind_String==Info[StreamKind](__T("StreamKind"), Info_Name, Info_Text))
+            break;
+    }
+
+    if (StreamKind>=Stream_Max)
+        return Ztring();
+
+    ZtringList List;
+    for (size_t FieldPos = 0; FieldPos < Info[StreamKind].size(); FieldPos++)
+        if (Info_Options < Info[StreamKind][FieldPos].size()
+         && InfoOption_ShowInXml < Info[StreamKind][FieldPos][Info_Options].size()
+         && Info[StreamKind][FieldPos][Info_Options][InfoOption_ShowInXml]==__T('Y'))
+            List.push_back(Xml_Name_Escape_0_7_78(Info[StreamKind][FieldPos][Info_Name]));
+
+    List.Separator_Set(0, __T(","));
+    return List.Read();
+}
+#endif // MEDIAINFO_ADVANCED
+
 //***************************************************************************
 // SubFile
 //***************************************************************************
@@ -2138,7 +2403,15 @@ void MediaInfo_Config::Event_Send (const int8u* Data_Content, size_t Data_Size,
     CriticalSectionLocker CSL(CS);
 
     if (Event_CallBackFunction)
+    {
+        MEDIAINFO_DEBUG1(   "Event",
+                            Debug+=", EventID=";Debug+=Ztring::ToZtring(LittleEndian2int32u(Data_Content), 16).To_UTF8();)
+
         Event_CallBackFunction ((unsigned char*)Data_Content, Data_Size, Event_UserHandler);
+
+        MEDIAINFO_DEBUG2(   "Event",
+                            )
+    }
 }
 #endif //MEDIAINFO_EVENTS
 
