@@ -1,21 +1,8 @@
-// File_Mpeg4v - Info for MPEG-4 Visual files
-// Copyright (C) 2006-2012 MediaArea.net SARL, Info@MediaArea.net
-//
-// This library is free software: you can redistribute it and/or modify it
-// under the terms of the GNU Library General Public License as published by
-// the Free Software Foundation, either version 2 of the License, or
-// any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-// GNU Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public License
-// along with this library. If not, see <http://www.gnu.org/licenses/>.
-//
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+/*  Copyright (c) MediaArea.net SARL. All Rights Reserved.
+ *
+ *  Use of this source code is governed by a BSD-style license that can
+ *  be found in the License.html file in the root of the source tree.
+ */
 
 //---------------------------------------------------------------------------
 // Pre-compilation
@@ -34,7 +21,7 @@
 //***************************************************************************
 
 //---------------------------------------------------------------------------
-#if defined(MEDIAINFO_MPEG4V_YES) || defined(MEDIAINFO_MPEG4_YES)
+#if defined(MEDIAINFO_MPEG4V_YES) || defined(MEDIAINFO_MPEG4_YES) || defined(MEDIAINFO_MXF_YES)
 //---------------------------------------------------------------------------
 
 #if MEDIAINFO_EVENTS
@@ -257,6 +244,7 @@ File_Mpeg4v::File_Mpeg4v()
 
     //Temp
     video_object_layer_start_IsParsed=false;
+    colour_description=false;
 }
 
 //---------------------------------------------------------------------------
@@ -292,11 +280,11 @@ bool File_Mpeg4v::Synched_Test()
     if (!Header_Parser_QuickSearch())
         return false;
 
-    #if MEDIAINFO_IBI
+    #if MEDIAINFO_IBIUSAGE
         bool RandomAccess=Buffer[Buffer_Offset+3]==0xB0; //SequenceHeader
         if (RandomAccess)
             Ibi_Add();
-    #endif //MEDIAINFO_IBI
+    #endif //MEDIAINFO_IBIUSAGE
 
     //We continue
     return true;
@@ -421,7 +409,7 @@ void File_Mpeg4v::Streams_Fill()
              if (aspect_ratio_info==0x02) PixelAspectRatio_Value=(float32)12/(float32)11;
         else if (aspect_ratio_info==0x03) PixelAspectRatio_Value=(float32)10/(float32)11;
         else if (aspect_ratio_info==0x04) PixelAspectRatio_Value=(float32)16/(float32)11;
-        else if (aspect_ratio_info==0x05) PixelAspectRatio_Value=(float32)40/(float32)13;
+        else if (aspect_ratio_info==0x05) PixelAspectRatio_Value=(float32)40/(float32)33;
         else if (aspect_ratio_info==0x0F && par_height) PixelAspectRatio_Value=((float32)par_width)/par_height;
         Fill(Stream_Video, 0, Video_PixelAspectRatio, PixelAspectRatio_Value, 3, true);
         Fill(Stream_Video, StreamPos_Last, Video_DisplayAspectRatio, ((float)object_layer_width)/object_layer_height*PixelAspectRatio_Value, 3, true);
@@ -430,11 +418,12 @@ void File_Mpeg4v::Streams_Fill()
     Fill(Stream_Video, 0, Video_BitDepth, bits_per_pixel);
     if (chroma_format<4)
         Fill(Stream_Video, 0, Video_Colorimetry, Mpeg4v_Colorimetry[chroma_format]);
-    if (colour_primaries!=(int8u)-1)
+    if (colour_description)
     {
-        Fill(Stream_Video, 0, "colour_primaries", Mpegv_colour_primaries(colour_primaries));
-        Fill(Stream_Video, 0, "transfer_characteristics", Mpegv_transfer_characteristics(transfer_characteristics));
-        Fill(Stream_Video, 0, "matrix_coefficients", Mpegv_matrix_coefficients(matrix_coefficients));
+        Fill(Stream_Video, 0, Video_colour_description_present, "Yes");
+        Fill(Stream_Video, 0, Video_colour_primaries, Mpegv_colour_primaries(colour_primaries));
+        Fill(Stream_Video, 0, Video_transfer_characteristics, Mpegv_transfer_characteristics(transfer_characteristics));
+        Fill(Stream_Video, 0, Video_matrix_coefficients, Mpegv_matrix_coefficients(matrix_coefficients));
     }
     if (low_delay)
     {
@@ -588,14 +577,14 @@ void File_Mpeg4v::Streams_Finish()
     {
         int32u Duration=(Time_End_Seconds-Time_Begin_Seconds)*1000+Time_End_MilliSeconds-Time_Begin_MilliSeconds;
         if (fixed_vop_time_increment && vop_time_increment_resolution)
-            Duration+=(float32_int32s)(((float)1000)/(((float)vop_time_increment_resolution)/fixed_vop_time_increment));
+            Duration+=float32_int32s(((float)1000)/(((float)vop_time_increment_resolution)/fixed_vop_time_increment));
         Fill(Stream_Video, 0, Video_Duration, Duration);
     }
 
-    #if MEDIAINFO_IBI
+    #if MEDIAINFO_IBIUSAGE
         if (fixed_vop_time_increment)
             Ibi_Stream_Finish(vop_time_increment_resolution, fixed_vop_time_increment);
-    #endif //MEDIAINFO_IBI
+    #endif //MEDIAINFO_IBIUSAGE
 }
 //***************************************************************************
 // Buffer - Demux
@@ -637,7 +626,9 @@ bool File_Mpeg4v::Demux_UnpacketizeContainer_Test()
                         case 0xB3 :
                         case 0xB6 :
                                     MustBreak=true; break;
-                        default   : MustBreak=false;
+                        default   :
+                                    Demux_Offset+=3;
+                                    MustBreak=false;
                     }
                     if (MustBreak)
                         break; //while() loop
@@ -656,9 +647,16 @@ bool File_Mpeg4v::Demux_UnpacketizeContainer_Test()
 
         if (!Status[IsAccepted])
         {
-            Accept("MPEG-4 Visual");
             if (Config->Demux_EventWasSent)
                 return false;
+            File_Mpeg4v* MI=new File_Mpeg4v;
+            Element_Code=(int64u)-1;
+            Open_Buffer_Init(MI);
+            Open_Buffer_Continue(MI, Buffer, Buffer_Size);
+            bool IsOk=MI->Status[IsAccepted];
+            delete MI;
+            if (!IsOk)
+                return false;
         }
 
         Demux_UnpacketizeContainer_Demux(Buffer[Buffer_Offset+3]==0xB0);
@@ -943,7 +941,7 @@ void File_Mpeg4v::video_object_layer_start()
             if (sprite_enable!=2) //No GMC
                 Skip_SB(                                        "low_latency_sprite_enable");
         }
-        if (video_object_layer_verid==1 && shape!=0) //Shape!=Rectangular
+        if (video_object_layer_verid!=1 && shape!=0) //Shape!=Rectangular
             Skip_SB(                                            "sadct_disable");
         TEST_SB_SKIP(                                           "bits_per_pixel_not_8_bit");
             Skip_S1(4,                                          "quant_precision");
@@ -1132,7 +1130,7 @@ void File_Mpeg4v::video_object_layer_start()
     BS_End();
 
     //Coherancy
-    if (object_layer_width==0 || object_layer_height==0 || object_layer_width/object_layer_height<0.1 || object_layer_width/object_layer_height>10)
+    if (object_layer_width==0 || object_layer_height==0 || ((float32)object_layer_width)/object_layer_height<((float32)0.1) || object_layer_width/object_layer_height>10)
         Trusted_IsNot("Problem with width and height!");
 
     FILLING_BEGIN();
@@ -1154,7 +1152,7 @@ void File_Mpeg4v::video_object_layer_start()
         video_object_layer_start_IsParsed=true;
         if (!Status[IsAccepted])
             Accept("MPEG-4 Visual");
-    FILLING_END()
+    FILLING_END();
 }
 
 //---------------------------------------------------------------------------
@@ -1422,7 +1420,7 @@ void File_Mpeg4v::visual_object_start()
         TEST_SB_SKIP(                                           "video_signal_type");
             Skip_S1(3,                                          "video_format");
             Skip_SB(                                            "video_range");
-            TEST_SB_SKIP(                                       "colour_description");
+            TEST_SB_GET (  colour_description,                  "colour_description");
                 Get_S1 (8, colour_primaries,                    "colour_primaries"); Param_Info1(Mpegv_colour_primaries(colour_primaries));
                 Get_S1 (8, transfer_characteristics,            "transfer_characteristics"); Param_Info1(Mpegv_transfer_characteristics(transfer_characteristics));
                 Get_S1 (8, matrix_coefficients,                 "matrix_coefficients"); Param_Info1(Mpegv_matrix_coefficients(matrix_coefficients));
